#!/bin/sh -e
#
# (c) Copyright 2016 Ultimaker B.V.
# Author: Olliver Schinagl <o.schinagl@ultimaker.com>
#
# SPDX-License-Identifier:	AGPL-3.0+
#
# Script to install the Ulti OS to a block device.
# TODO: The script just stops on failure, errors should be trapped so things
#	can be properly cleaned up.

MBR_BOOTLOADER_START=2048 # 1 MiB offset is a common and sane default.
MBR_BOOTLOADER_SIZE=16384 # 8 MiB
MBR_ROOTFS_START=$(expr ${MBR_BOOTLOADER_START} + ${MBR_BOOTLOADER_SIZE})
MBR_ROOTFS_SIZE=$(expr 999424 - ${MBR_ROOTFS_START}) # ~480 MiB (aligned) - Bootloader. So that it should in theory even fit 512 MB storage size (e.g. 1 M = 1000000 bytes rather then 1048576 bytes)
MBR_USER_START=$(expr ${MBR_ROOTFS_START} + ${MBR_ROOTFS_SIZE})
# XXX we always assume that the eMMC is the second device. This fails if:
# - We launch the installer via tftp(net), the target dev is then /dev/mmcblk0.
# - An additional mmc interface is added on bus mmc1
# - The bootloader/dts changed and re-arranges the device
# - It probably is wiser to supply this to the kernel via a cmdline arg?
TARGET_DEV=/dev/mmcblk1
TARGET_ROOT=/tmp/install_root

UBOOTENV=um_u-boot.env.bin

ROOTFS_TXZ=/ulti_installer/rootfs.tar.xz
BOM_NUMBER_FILE=/ulti_installer/bom.txt

# Very ugly ugly hack to get text on the display. This is untested in combination with a secondary fb (hdmi)
dmesg -n 1 || true
modprobe fbcon || true

led_off() {
	if [ -z "${1}" ]; then
		return
	fi
	case "${1}" in
		"0" )
			echo "none" > /sys/class/leds/opinicus:green:usr/trigger || true
			;;
		"1" )
			echo "none" > /sys/class/leds/opinicus:orange:d1/trigger || true
			echo "none" > /sys/class/leds/opinicus:orange:ol1/trigger || true
			;;
		"2" )
			echo "none" > /sys/class/leds/opinicus:orange:d2/trigger || true
			echo "none" > /sys/class/leds/opinicus:orange:ol2/trigger || true
			;;
		"3" )
			echo "none" > /sys/class/leds/opinicus:orange:ol3/trigger || true
			;;
		"button_ring" )
			echo "none" > /sys/class/leds/pca963x:red/trigger || true
			echo "none" > /sys/class/leds/pca963x:red:button_ring/trigger || true
			echo "none" > /sys/class/leds/pca963x:green/trigger || true
			echo "none" > /sys/class/leds/pca963x:green:button_ring/trigger || true
			echo "none" > /sys/class/leds/pca963x:blue/trigger || true
			echo "none" > /sys/class/leds/pca963x:blue:button_ring/trigger || true
			;;
		"cabin_light" )
			echo "none" > /sys/class/leds/pca963x:red:cabin_light/trigger || true
			echo "none" > /sys/class/leds/pca963x:green:cabin_light/trigger || true
			echo "none" > /sys/class/leds/pca963x:blue:cabin_light/trigger || true
			echo "none" > /sys/class/leds/pca963x:white:cabin_light/trigger || true
			;;
	esac
}

led_on() {
	if [ -z "${1}" ]; then
		return
	fi
	case "${1}" in
		"0" )
			echo "default-on" > /sys/class/leds/opinicus:green:usr/trigger || true
			;;
		"1" )
			echo "default-on" > /sys/class/leds/opinicus:orange:d1/trigger || true
			echo "default-on" > /sys/class/leds/opinicus:orange:ol1/trigger || true
			;;
		"2" )
			echo "default-on" > /sys/class/leds/opinicus:orange:d2/trigger || true
			echo "default-on" > /sys/class/leds/opinicus:orange:ol2/trigger || true
			;;
		"3" )
			echo "default-on" > /sys/class/leds/opinicus:orange:ol3/trigger || true
			;;
		"button_ring" )
			echo "default-on" > /sys/class/leds/pca963x:red/trigger || true
			echo "default-on" > /sys/class/leds/pca963x:red:button_ring/trigger || true
			echo "default-on" > /sys/class/leds/pca963x:green/trigger || true
			echo "default-on" > /sys/class/leds/pca963x:green:button_ring/trigger || true
			echo "default-on" > /sys/class/leds/pca963x:blue/trigger || true
			echo "default-on" > /sys/class/leds/pca963x:blue:button_ring/trigger || true
			;;
		"cabin_light" )
			echo "default-on" > /sys/class/leds/pca963x:red:cabin_light/trigger || true
			echo "default-on" > /sys/class/leds/pca963x:green:cabin_light/trigger || true
			echo "default-on" > /sys/class/leds/pca963x:blue:cabin_light/trigger || true
			echo "default-on" > /sys/class/leds/pca963x:white:cabin_light/trigger || true
			;;
	esac
}

led_blink() {
	if [ -z "${1}" ]; then
		return
	fi
	if [ -z "${2}" ]; then
		return
	fi
	case "${1}" in
		"0" )
			echo "timer" > /sys/class/leds/opinicus:green:usr/trigger || true
			echo "${2}" > /sys/class/leds/opinicus:green:usr/delay_off || true
			echo "${2}" > /sys/class/leds/opinicus:green:usr/delay_on || true
			;;
		"1" )
			echo "timer" > /sys/class/leds/opinicus:orange:d1/trigger || true
			echo "${2}" > /sys/class/leds/opinicus:orange:d1/delay_off || true
			echo "${2}" > /sys/class/leds/opinicus:orange:d1/delay_on || true
			echo "timer" > /sys/class/leds/opinicus:orange:d1/trigger || true
			echo "${2}" > /sys/class/leds/opinicus:orange:ol1/delay_off || true
			echo "${2}" > /sys/class/leds/opinicus:orange:ol1/delay_on || true
			;;
		"2" )
			echo "timer" > /sys/class/leds/opinicus:orange:d2/trigger || true
			echo "${2}" > /sys/class/leds/opinicus:orange:d2/delay_off || true
			echo "${2}" > /sys/class/leds/opinicus:orange:d2/delay_on || true
			echo "timer" > /sys/class/leds/opinicus:orange:ol2/trigger || true
			echo "${2}" > /sys/class/leds/opinicus:orange:ol2/delay_off || true
			echo "${2}" > /sys/class/leds/opinicus:orange:ol2/delay_on || true
			;;
		"3" )
			echo "timer" > /sys/class/leds/opinicus:orange:ol3/trigger || true
			echo "${2}" > /sys/class/leds/opinicus:orange:ol3/delay_off || true
			echo "${2}" > /sys/class/leds/opinicus:orange:ol3/delay_on || true
			;;
		"button_ring" )
			echo "timer" > /sys/class/leds/pca963x:red/trigger || true
			echo "timer" > /sys/class/leds/pca963x:red:button_ring/trigger || true
			echo "timer" > /sys/class/leds/pca963x:green/trigger || true
			echo "timer" > /sys/class/leds/pca963x:green:button_ring/trigger || true
			echo "timer" > /sys/class/leds/pca963x:blue/trigger || true
			echo "timer" > /sys/class/leds/pca963x:blue:button_ring/trigger || true
			;;
		"cabin_light" )
			echo "timer" > /sys/class/leds/pca963x:red:cabin_light/trigger || true
			echo "timer" > /sys/class/leds/pca963x:green:cabin_light/trigger || true
			echo "timer" > /sys/class/leds/pca963x:blue:cabin_light/trigger || true
			echo "timer" > /sys/class/leds/pca963x:white:cabin_light/trigger || true
			;;
	esac
}

led_cabin_light_color_set() {
	if [ -z "${1}" ]; then
		return
	fi
	if [ -z "${2}" ]; then
		return
	fi
	if [ -z "${3}" ]; then
		return
	fi

	echo "${1}" > /sys/class/leds/pca963x:red:cabin_light/brightness || true
	echo "${2}" > /sys/class/leds/pca963x:green:cabin_light/brightness || true
	echo "${3}" > /sys/class/leds/pca963x:blue:cabin_light/brightness || true
	echo "${4}" > /sys/class/leds/pca963x:white:cabin_light/brightness || true
}

led_button_ring_color_set() {
	if [ -z "${1}" ]; then
		return
	fi
	if [ -z "${2}" ]; then
		return
	fi
	if [ -z "${3}" ]; then
		return
	fi

	echo "${1}" > /sys/class/leds/pca963x:red/brightness || true
	echo "${1}" > /sys/class/leds/pca963x:red:button_ring/brightness || true
	echo "${2}" > /sys/class/leds/pca963x:green/brightness || true
	echo "${2}" > /sys/class/leds/pca963x:green:button_ring/brightness || true
	echo "${3}" > /sys/class/leds/pca963x:blue/brightness || true
	echo "${3}" > /sys/class/leds/pca963x:blue/button_ring/brightness || true
}

led_blink 0 500
led_on 1
led_off 2
led_off "button_ring"
led_button_ring_color_set 0 0 255
led_blink "button_ring" 500

exit_installer() {
	echo 0 > /sys/class/vtconsole/vtcon1/bind || true
	rmmod fbcon || true

	led_off 0
	led_blink 1 500
	led_on 2
	led_button_ring_color_set 0 255 0
	led_cabin_light_color_set 0 255 0 0

	exit
}

exit_installer_error() {
	echo 0 > /sys/class/vtconsole/vtcon1/bind || true
	rmmod fbcon || true

	led_blink 0 50
	led_blink 1 50
	led_blink 2 50
	led_button_ring_color_set 255 0 0
	led_blink "button_ring" 50
	led_cabin_light_color_set 255 0 0 0
	led_blink "cabin_light"

	exit 1
}
# Setup the default exit handler to show an error on all possible indicators.
trap exit_installer_error EXIT

clear_fb()
{
	# Warning, if we write more to the display then what the display can handle, we get into an eternal loop.
	# With the current font, we have 32x10 characters on the display and thus writing 319 spaces works.
	# This would need to be improved.
	if [ -n ${1} ]; then
		num_chars=319
	else
		num_chars=${1}
	fi
	fb=""
	for i in `seq ${num_chars}`
	do
		fb="${fb} "
	done
	echo "${fb}" | tee /dev/vcs
}

print_fb()
{
	# Wipe the entire screen first
	clear_fb
	# Skip the first two lines
	fb=""
	for i in `seq 64`
	do
		fb="${fb} "
	done
	echo "$fb${1}" | tee /dev/vcs
}

INSTALLER_VERSION="Unknown version!"
if [ -f /etc/ultimaker_version ]; then
	INSTALLER_VERSION=$(cut -c 1-32 /etc/ultimaker_version)
fi

print_fb "01/15 | Installation of UltiOS  Installer version:              ${INSTALLER_VERSION}"
if [ -z ${1+x} ]; then
	print_fb "Warning: No rootfs supplied,    falling back to: ${ROOTFS_TXZ}."
else
	ROOTFS_TXZ=${1}
fi
if [ ! -f ${ROOTFS_TXZ} ]; then
	print_fb "Error: no rootfs archive found. Aborting."
	exit_installer
fi

print_fb "02/15 | Partitioning eMMC"

/sbin/sfdisk -uS ${TARGET_DEV} <<-EOT && true
${MBR_BOOTLOADER_START},${MBR_BOOTLOADER_SIZE},L
${MBR_ROOTFS_START},${MBR_ROOTFS_SIZE},L
${MBR_USER_START},,L
EOT

# Run partprobe, sometimes sfdisk fails, and then running partprobe will fix the partition tables known by the kernel.
partprobe ${TARGET_DEV}

# Unmount partitions first, but don't wait (for partitions being active) while ignoring errors (partitions not being mounted)
umount -l ${TARGET_DEV}p3 || true
umount -l ${TARGET_DEV}p1 || true
umount -l ${TARGET_DEV}p2 || true

print_fb "03/15 | Formatting eMMC boot"
mkfs.ext4 -F ${TARGET_DEV}p1 -L boot -O ^extents,^64bit
print_fb "04/15 | Formatting eMMC root"
mkfs.f2fs ${TARGET_DEV}p2 -l root
print_fb "05/15 | Formatting eMMC user"
mkfs.f2fs ${TARGET_DEV}p3 -l user

# Sleeping 5 seconds, as it takes time for the /dev/disk/by-label to be updated, and else the mount after this will fail.
sleep 5

print_fb "06/15 | Mounting eMMC root"
mkdir -p ${TARGET_ROOT}
mount /dev/disk/by-label/root ${TARGET_ROOT}
print_fb "07/15 | Mounting eMMC boot"
mkdir -p ${TARGET_ROOT}/boot
mount /dev/disk/by-label/boot ${TARGET_ROOT}/boot

print_fb "08/15 | Extracting rootfs ..."
cd ${TARGET_ROOT}
tar --warning=none -xf ${ROOTFS_TXZ}

if [ -f ${BOM_NUMBER_FILE} ]; then
    print_fb "09/15 | Allocating MAC address, and programming BOM number"
    BOM_NUMBER=$(cat ${BOM_NUMBER_FILE})
    /usr/bin/python3 /ulti_installer/configure_eeprom.py ${BOM_NUMBER}
fi

print_fb "10/15 | Copying bootloader files"
dd if=${TARGET_ROOT}/boot/u-boot-sunxi-with-spl.bin of=${TARGET_DEV} bs=1024 seek=8 conv=notrunc,fsync
if [ -f ${TARGET_ROOT}/boot/${UBOOTENV} ]; then
    dd if=${TARGET_ROOT}/boot/${UBOOTENV} of=${TARGET_DEV} bs=1024 seek=544 count=128 conv=notrunc,fsync
fi
cp ${TARGET_ROOT}/boot/boot_emmc.scr /${TARGET_ROOT}/boot/boot.scr

print_fb "11/15 | Prepping user storage"
mkdir -p ${TARGET_ROOT}/home/ultimaker/tmp # TODO find a proper place to mount
mount /dev/disk/by-label/user home/ultimaker/tmp
chown ultimaker:ultimaker -R home/ultimaker

print_fb "12/15 | Writing fstab"
cat >> etc/fstab <<-EOT
LABEL=boot /boot ext4 defaults,noauto 0 1
LABEL=user /home/ultimaker/tmp f2fs defaults 0 0
EOT
cd /

print_fb "13/15 | Unmounting storage"
sync
umount -l /dev/disk/by-label/user
umount -l /dev/disk/by-label/boot
umount -l /dev/disk/by-label/root
rmdir ${TARGET_ROOT}

print_fb "14/15 | Finishing ..."

print_fb "15/15 | Installation complete   Please power off now and        remove the microSD card"

# We are done, replace our exit handler with the one that shows success.
trap exit_installer EXIT
