from typing import Optional

MYPY = False
if MYPY:
    from app.models.printer import Printer


class PrinterCredentialsStore:
    def __init__(self, *, printer: "Optional[Printer]"=None) -> None:
        if printer:
            self.printer = printer
            # The id of the authorization, this equals the username for the auhorization with the printer.
            self.id = printer.digest_auth_id
            # The "key" part of the authorization, this equals the password for the authorization with the printer.
            # Do not expose this on APIs, in logs on in HTTP requests. Keep it internal.
            self.key = printer.digest_auth_key
            # Flag to indicate if this auth key/id pair is already accepted by the end user on the machine
            self.authorized = printer.digest_auth_authorized
        else:
            self.printer = None
            self.__empty()

    def write(self):
        if self.printer is not None:
            with self.printer.atomic_modification:
                self.printer.digest_auth_id = self.id
                self.printer.digest_auth_key = self.key
                self.printer.digest_auth_authorized = self.authorized

    def reset(self):
        self.__empty()
        self.write()

    def __empty(self):
        self.id = None
        self.key = None
        self.authorized = False
