from typing import Callable, TypeVar, Generic
from pyrsistent import pvector

from um3_commons import thread

T = TypeVar('T')


class Signal(Generic[T]):
    """A simple publish/subscribe signal system
    """
    def __init__(self) -> None:
        self.__connections = pvector()
        self.__lock = thread.RLock()

    def connect(self, connector: Callable[[T], None]) -> None:
        with self.__lock:
            self.__connections = self.__connections.append(connector)

    def disconnect(self, connector: Callable[[T], None]) -> None:
        with self.__lock:
            if connector not in self.__connections:
                return  # Ignore unconnected connector disconnects
            self.__connections = self.__connections.remove(connector)

    def clear_connections(self):
        with self.__lock:
            self.__connections = pvector()

    def emit(self, payload: T=None) -> None:
        with self.__lock:
            connections = self.__connections

        for connection in connections:
            if payload is None:
                connection()
            else:
                connection(payload)
