import xml.etree.ElementTree as ET

from typing import Any, Dict, Iterator


class Material:
    EMPTY_MATERIAL = None       # type: Material
    UNKNOWN_MATERIAL = None     # type: Material

    def __init__(self, guid, brand, material, color) -> None:
        self.guid = guid
        self.brand = brand
        self.material = material
        self.color = color

    def __iter__(self) -> Iterator[str]:
        return iter({
            "guid": self.guid,
            "brand": self.brand,
            "material": self.material,
            "color": self.color
        })

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, Material):
            return self.guid == other.guid
        return False

    @staticmethod
    def __parse_material_metadata(xml: str) -> Dict[str, str]:
        root = ET.fromstring(xml)
        namespace = "{http://www.ultimaker.com/material}"
        material_metadata = {}  # type: Dict[str, str]

        for fdmmaterial in root.iter(namespace + "fdmmaterial"):
            for metadata in fdmmaterial.iter(namespace + "metadata"):
                for guid in metadata.iter(namespace + "GUID"):
                    material_metadata["GUID"] = guid.text

                for name in metadata.iter(namespace + "name"):
                    for child in name:
                        property_name = child.tag[len(namespace):]
                        if property_name in ["brand", "material", "color"]:
                            material_metadata[property_name] = child.text

        return material_metadata

    @property
    def pretty_string(self) -> str:
        if self.material == "empty":
            return "Empty"
        if self.material and self.color:
            if self.color == "Generic":
                return self.material
            elif self.color == "unknown":
                return "Unknown"
            return "%s %s" % (self.color, self.material)
        elif self.material:
            return "%s" % (self.material)
        else:
            return str(self)

    def __str__(self) -> str:
        return "Material(%s, %s, %s, %s)" % (self.guid, self.brand, self.material, self.color)

Material.EMPTY_MATERIAL = Material("", "", "empty", "")
Material.UNKNOWN_MATERIAL = Material("", "", "unknown", "")
