from typing import Any, Dict, Union

from app.constants.print_job_status import PrintJobStatus


class PrintJob:
    def __init__(self, data: Dict[str, Union[str, int]]) -> None:
        self.name = str(data.get("name", ""))
        self.uuid = str(data["uuid"]).lower()    # type: str
        self.state = self.__parse_state(data)
        self.time_elapsed = int(data.get("time_elapsed", 0))    # type: int
        self.time_total = int(data.get("time_total", 0))        # type: int
        if self.state not in PrintJobStatus.ALL:
            raise RuntimeError("Invalid state '%s" % self.state)
        self.source = str(data.get("source", ""))
        self.source_user = str(data.get("source_user", ""))
        self.reprint_original_uuid = str(data.get("reprint_original_uuid", ""))

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, PrintJob):
            return self.uuid == other.uuid and self.state == other.state

        return False

    @staticmethod
    def __parse_state(data: Dict[str, Union[str, int]]) -> str:
        if "state" in data:
            state = str(data["state"])
        else:
            state = str(data["result"])

        return state.lower()

    def __str__(self) -> str:
        return "UM3PrintJob(%s,%s)" % (self.uuid, self.state)
