from typing import List

import logging

from app.models.abstraction.abstract_collection_model import AbstractCollectionModel
from app.models.print_job import PrintJob

logger = logging.getLogger(__name__)


class PrintJobCollection(AbstractCollectionModel[PrintJob]):
    def __init__(self) -> None:
        super().__init__()
        self.onDelete.connect(lambda print_job: logger.info("Removed print_job[%s]", print_job))
        self.onAdd.connect(lambda print_job: logger.info("Added print_job[%s]", print_job))
        self.onMove.connect(lambda payload: logger.info("Moved print_job[%s] to %s", *payload))

    def all(self) -> List[PrintJob]:
        return list(self.data.values())

    def queued(self) -> List[PrintJob]:
        queued_jobs = filter(lambda print_job: not print_job.started, self.all())
        return list(queued_jobs)

    def printing(self) -> List[PrintJob]:
        printing_jobs = filter(lambda print_job: print_job.started, self.all())
        return list(printing_jobs)
