import logging

from app.libs.ultimaker3.material import Material

try:
    import dbus
    from um3_commons.dbusif import RemoteObject
except Exception:
    RemoteObject = None

logger = logging.getLogger(__name__)


class MaterialManager:
    def __init__(self, use_dbus: bool) -> None:
        if use_dbus and RemoteObject is not None:
            self.__material_service = RemoteObject("material")
        else:
            self.__material_service = None

    def get(self, guid: str) -> Material:
        if self.__material_service is None:
            return Material("506c9f0d-e3aa-4bd4-b2d2-23e2425b1aa8", "DebugFab", "PLA", "Chocolate Brown")

        if guid == "":
            # Empty string is a valid guid for no material selected in the printer.
            return Material.EMPTY_MATERIAL

        material_data = self.__material_service.getMaterialNameFields(guid)
        if "?" in material_data:
            logger.warning("Unable to find material data for GUID [%s]", guid)
            return Material.UNKNOWN_MATERIAL

        return Material(guid, *material_data)
