from typing import Dict, Any, List, Tuple

from app.services.material_manager import MaterialManager
from app.services.scheduler.result_containers.match_results import ConfigurationChangeMatchResult
from app.services.scheduler.result_containers.match_results_containers import MatchResultListPerPrinter


class ConfigurationMisMatchStringifier:
    def __init__(self, material_manager: MaterialManager) -> None:
        self.__material_manager = material_manager

    # this function generates one string to request material changes to be sent to the display of the printer.
    def stringify(self, match_results: MatchResultListPerPrinter) -> str:
        changes = self.combine_and_name_changes(match_results)
        text_changes = []  # type:List[Tuple[int, str]]

        for change in changes:
            if change.type_of_change in ["material_change", "material_insert"]:
                text_changes.append((change.index, "Material %s:\n%s" % (change.index + 1, change.target_name)))
            elif change.type_of_change == "buildplate_change":
                text_changes.append((99, "Build plate: %s" % (change.target_name)))
            else:
                text_changes.append((change.index + 2, "PrintCore %s: %s" % (change.index + 1, change.target_name)))

        result_string = ["Change configuration"]  # type: List[str]
        for index, print_core in sorted(text_changes):
            result_string.append(print_core)

        return "\n".join(result_string)

    # This function gets all the changes required from the match result and finds a friendly name for each of the
    # materials.
    def combine_and_name_changes(self, match_results: MatchResultListPerPrinter) -> List[ConfigurationChangeMatchResult.ChangeSet]:
        output = []  # type: List[ConfigurationChangeMatchResult.ChangeSet]

        for result in match_results.results:
            if isinstance(result, ConfigurationChangeMatchResult):
                change = result.change
                if change.type_of_change in ["material_change", "material_insert"]:
                    target_material = self.__material_manager.get(change.target_id)
                    origin_material = self.__material_manager.get(change.origin_id)
                    change.origin_name = origin_material.pretty_string if origin_material is not None else "Unknown"
                    change.target_name = target_material.pretty_string if target_material is not None else "Unknown"
                else:
                    change.target_name = change.target_id
                    change.origin_name = change.origin_id

                output.append(change)
        return output
