from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult, MatchResult
from um3_commons.math.axisAlignedBox import AxisAlignedBox


class FitsInBuildVolumeMatchRule(MatchRule):
    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        printer_volume = printer.machine_variant.print_volume
        if printer_volume.intersects_box(print_job.boundary_box) != AxisAlignedBox.IntersectionResult.FullIntersection:
            return IncompatibleMatchResult(
                origin=self,
                debug_message="The print job does not fit inside the build volume of the printer",
                translation_key="does_not_fit_in_build_volume"
            )
        return None
