from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult, MatchResult, Severity


class HasNotFailedOnPrinterMatchRule(MatchRule):
    MAX_TIES = 2

    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        tries = print_job.get_start_error_count_with_printer(printer)
        if tries >= self.MAX_TIES:
            return IncompatibleMatchResult(
                origin=self,
                debug_message="Can not print on this printer because it failed to often ({})".format(tries),
                translation_key="has_failed_too_often_on_printer",
                severity=Severity.USER_FIXABLE
            )
        return None
