from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import ConfigurationChangeMatchResult
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult
from app.services.scheduler.result_containers.match_results import MatchResult


class IsBuildplateCompatibleMatchRule(MatchRule):
    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        if printer.configuration.is_empty():
            return IncompatibleMatchResult(
                origin=self,
                debug_message="The printer does not have any configuration yet",
                translation_key="loading_config"
            )

        if print_job.build_plate.type != printer.build_plate.type:
            return ConfigurationChangeMatchResult(
                change=ConfigurationChangeMatchResult.ChangeSet(
                    type_of_change="buildplate_change",
                    origin_id=printer.build_plate.type,
                    target_id=print_job.build_plate.type),
                origin=self,
                debug_message="buildplate type mismatch: printer has {}, job expects {}".format(
                    printer.build_plate.type, print_job.build_plate.type),
                translation_key="buildplate_change"
            )
        return None
