from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult, MatchResult


class MatchesConstrainedPrinterNameRule(MatchRule):
    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        constrained_name = print_job.constraints.get("require_printer_name")
        if constrained_name is not None and printer.unique_name != constrained_name:
            return IncompatibleMatchResult(
                origin=self,
                debug_message="This job requires printer name {} instead of {}".format(constrained_name, printer.unique_name),
                translation_key="constrained_to_other_printer_name"
            )
        return None
