from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import ConfigurationChangeMatchResult, MatchResult


class OverruleConfigurationMismatch(MatchRule):
    def __init__(self, filter_rule: MatchRule) -> None:
        self.__filter_rule = filter_rule

    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        result = self.__filter_rule.apply(printer, print_job)
        constraint = print_job.constraints.get("require_printer_name")
        # only allow a ConfigurationChangeMatchResult to be ignored if the force flag is set and the printer is known!
        if print_job.force and isinstance(result, ConfigurationChangeMatchResult) and constraint is not None:
            return None
        return result
