from typing import List, Optional

from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult, MatchResult


class RuleStack(MatchRule):
    def __init__(self, *stack: MatchRule) -> None:
        self.__rule_stack = stack

    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        for match_rule in self.__rule_stack:
            result = None
            try:
                result = match_rule.apply(printer, print_job)
            except Exception as e:
                result = IncompatibleMatchResult(
                    origin=match_rule,
                    debug_message="Exception in matcher: " + str(e),
                    translation_key="exception"
                )

            if result is not None:
                return result

        return None
