import fcntl
import array
import ctypes
import struct
import os


## Software bitbanging SPI bus
#  Can be used to talk SPI on any GPIO pins exported by linux.
class SpiBus():
    def __init__(self, device_name):
        self._avr_sck = os.open("%s_sck" % (device_name), os.O_WRONLY)
        self._avr_miso = os.open("%s_miso" % (device_name), os.O_RDONLY)
        self._avr_mosi = os.open("%s_mosi" % (device_name), os.O_WRONLY)

    # @param: do_read when set will return the read data. Setting this to False gives an extra speed improvement when the read data isn't required. 
    def _transferByte(self, b, do_read):
        pwrite = os.pwrite
        pread = os.pread
        avr_sck = self._avr_sck
        avr_miso = self._avr_miso
        avr_mosi = self._avr_mosi

        ret = 0
        for n in range(8):
            if (b & (0x80 >> n)):
                pwrite(avr_mosi, b"1", 0)
            else:
                pwrite(avr_mosi, b"0", 0)

            # Clock pulse
            pwrite(avr_sck, b"0", 0)
            pwrite(avr_sck, b"1", 0)

            if do_read and pread(avr_miso, 1, 0) == b"1":
                ret |= (0x80 >> n)

        pwrite(avr_sck, b"0", 0)
        return ret

    ## Transfer the txbuff and read back the data from spi.
    #  @param txbuff, a bytes object containing the data to be transfered trough the SPI bus.
    #  @param: do_read will return the read data by default. Setting this to False gives an extra speed improvement when the read data isn't required. 
    #  @return a bytes object containing the result from the SPI transfer. This object will have the same length as the given txbuff.
    def transfer(self, tx_buffer, do_read=True):
        rx_buffer = bytearray(len(tx_buffer))
        for n in range(len(tx_buffer)):
            rx_buffer[n] = self._transferByte(tx_buffer[n], do_read)
        return rx_buffer
