from .lowLevelStk500v2 import LowLevelStk500v2
from ...common.firmwareUpdateBase import FirmwareUpdateBase

import logging

log = logging.getLogger(__name__.split(".")[-1])


## Stk500v2 based firmware update tool
#  Verified to be compatible with the Arduino bootloader, as well as our own bootloader (which is based on the Arduino one)
class Stk500v2(FirmwareUpdateBase):
    # Connect to the hardware. Returns false if it fails to connect.
    # @param kwargs: May contain:
    #       port: Required, serial port name (/dev/ttyS1, COM30)
    #       speed: Optional, serial speed (default: 115200
    #       reset: Optional, gpio device used instead for reading/writing the reset line. If not supplied the DTR line from the serial device is used.
    # @return: Low level
    def _connectLowLevel(self, **kwargs):
        try:
            return LowLevelStk500v2(**kwargs)
        except:
            log.exception("Failed to open low level stk500v2")
        return None


def main(filename):
    stk500v2 = Stk500v2()
    if stk500v2.connect(port="/dev/ttyS1", reset="/dev/gpio/avr_reset"):
        stk500v2.program(filename)
    stk500v2.close()
    return True

if __name__ == "__main__":
    logging.basicConfig(format="%(asctime)-15s %(levelname)-8s %(name)-15s %(message)s", level=logging.DEBUG)

    import sys
    for arg in sys.argv[1:]:
        main(sys.argv[1])
