from griffin.preferences.registry import Registry
from griffin.printer.controller import Controller


# @brief Class that initializes the header data
class GCodeHeaderData():
    # Define the currently used header version
    HEADER_VERSION = "0.1"
    # Define the currently used header flavor
    HEADER_FLAVOR = "Griffin"
    # Define a data unknown value for header parameters
    DATA_UNKNOWN = -1.0

    # @brief Constructs a non-shared empty header
    # @return Returns the default empty header as a registry
    @classmethod
    def getEmptyHeader(cls):
        header = {
            "HEADER_VERSION": GCodeHeaderData.DATA_UNKNOWN,  # no header parsed use old behaviour
            "FLAVOR": None,
            "GENERATOR": {"NAME": None, "VERSION": None, "BUILD_DATE": None},
            "EXTRUDER_TRAIN": {},
            "BUILD_PLATE": {"INITIAL_TEMPERATURE": None},
            "PRINT": {
                "TIME": GCodeHeaderData.DATA_UNKNOWN,  # same as time_total
                "SIZE": {
                    "MIN": {"X": -float("inf"), "Y": -float("inf"), "Z": -float("inf")},
                    "MAX": {"X": float("inf"), "Y": float("inf"), "Z": float("inf")}
                }
            }
        }

        for i in range(Controller.getInstance().getPropertyValue("hotend_count")):
            header["EXTRUDER_TRAIN"][(str(i))] = GCodeHeaderData.__getEmptyExtruderTrain()

        return Registry(header)

    # @brief Creates a non-shared empty extruder train
    # @return Returns the default empty extruder train as a dictionary
    @classmethod
    def __getEmptyExtruderTrain(cls):
        return {
            "INITIAL_TEMPERATURE": None,  # leave heater off when left out
            "MATERIAL": {
                "VOLUME_USED": None,
                "GUID": None
            },
            "NOZZLE": {"DIAMETER": None}
        }
