import inspect
import traceback
import warnings
import functools

import time

import logging

log = logging.getLogger(__name__.split(".")[-1])

## @brief deprecation marker that shows a warning for every usage
# This is a decorator which can be used to mark functions
# as deprecated. It will result in a warning being emitted
# when the function is used first time and filter is set for show DeprecationWarning.
#
# From http://stackoverflow.com/questions/2536307/decorators-in-the-python-standard-lib-deprecated-specifically
def deprecated(message: str = ''):
    def decorator_wrapper(func):
        @functools.wraps(func)
        def function_wrapper(*args, **kwargs):
            current_call_source = '|'.join(traceback.format_stack(inspect.currentframe()))
            if current_call_source not in function_wrapper.last_call_source:
                log.warning("Function {} is now deprecated! {}".format(func.__name__, message))
                warnings.warn("Function {} is now deprecated! {}".format(func.__name__, message),
                              category=DeprecationWarning, stacklevel=2)
                function_wrapper.last_call_source.add(current_call_source)

            return func(*args, **kwargs)

        function_wrapper.last_call_source = set()

        return function_wrapper
    return decorator_wrapper


if __name__ == "__main__":

    @deprecated('You must use my_func2!')
    def my_func():
        print('aaa')

    my_func()