from contextlib import contextmanager

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

STORAGE = 'sqlite:////var/lib/griffin/history.db'
engine = create_engine(STORAGE)
Session = sessionmaker(bind=engine)
Base = declarative_base()

Base.metadata.bind = engine

## @brief Context manager to open/commit/rollback/close database sessions
# Usage:
# >>> with session_scope() as session:
# >>>     session.doSomething
# @returns a SQLAlchemy database session that can be used for querying and updating the database
@contextmanager
def session_scope(*args, **kwargs):
    """Provide a transactional scope around a series of operations."""
    session = Session(*args, **kwargs)
    try:
        yield session
        session.commit()
    except:
        session.rollback()
        raise
    finally:
        session.close()
