import uuid

from griffin.datatypes.printJob import PrintJob
from griffin.history.alchemy.database_link import Base

from griffin.history.alchemy.uuidType import UuidType
from sqlalchemy import Column, String, Interval, DateTime

from datetime import timedelta


## @brief Database representation of a PrintJob
#  Note, if you modify this database layout, you need to build a migration file and increase the version number in the database.
#       See databaseService.py as start point for migrations.
class PrintJobRecord(Base):
    __tablename__ = "PrintJobs"

    # Id = Column(Integer, primary_key=True)

    uuid                  = Column(UuidType(), primary_key=True, default=uuid.uuid4) #UUID of the job
    name                  = Column(String) # name of the job
    source                = Column(String) # where did the job come from, who/what originated it
    time_elapsed          = Column(Interval) # how long has the job been printing already, including pauses
    datetime_started      = Column(DateTime) #, when was the job started?  Defaults to None
    datetime_finished     = Column(DateTime) #, when was the job finished?  Defaults to None
    datetime_cleaned      = Column(DateTime) #, when was the print resulting from this bed removed from the bed? Defaults to None
    result                = Column(String) # was the job finished successfully or not?
    time_estimated        = Column(Interval) # the estimated job time, retrieved from the g-code
    reprint_original_uuid = Column(UuidType())

    def __repr__(self):
        return "PrintJobRecord(uuid={uuid})".format(uuid=self.uuid)

    ## @brief Convert a PrintJobRecord to a 'native' PrintJob without any SqlAlchemy relation
    # @returns PrintJob
    def asNative(self):
        return PrintJob(uuid=self.uuid,
                        name=self.name,
                        state=None,
                        source=self.source,
                        reprint_original_uuid=self.reprint_original_uuid,
                        time_elapsed=self.time_elapsed,
                        time_total=None,
                        datetime_started=self.datetime_started,
                        datetime_finished=self.datetime_finished,
                        datetime_cleaned=self.datetime_cleaned,
                        result=self.result,
                        time_estimated = self.time_estimated if self.time_estimated is not None else timedelta())
