import json
import tokenize
import io


## Replacement for json.load
#  This function loads json data into python objects just like json.load
#  Which the following extra additions:
#  Comments are allowed in the json data, in the form of #COMMENT and //COMMENT
#  Extra commas inside arrays and dictionaries are accepted [1,2,3,] {"A":1,"B":2,}
#  Simply replacing "import json" with "from griffin import json"
#  @param: fp, filelike object, usually obtained by open()
#  @return json data inside the file.
def load(fp):
    json_data = []
    if hasattr(fp, "encoding"):
        readline = lambda : fp.readline().encode("utf-8")
    else:
        readline = fp.readline
    tokenizer = tokenize.tokenize(readline)
    for t in tokenizer:
        if t.type == tokenize.COMMENT or t.type == tokenize.ENCODING:
            continue
        if t.string == "//":
            while t.type != tokenize.NL and t.type != tokenize.NEWLINE and t.type != tokenize.ENDMARKER:
                t = tokenizer.__next__()
            continue
        if t.string == "}" or t.string == "]":
            n = -1
            while json_data[n].strip() == "":
                n -= 1
            if json_data[n] == ",":
                json_data.pop(n)
        json_data.append(t.string)
    return json.loads("".join(json_data))


## Replacement for json.loads
#  See load() for details.
#  @param s: string or bytes object containing json data.
#  @return json data inside the object.
def loads(s):
    if isinstance(s, bytes):
        return load(io.BytesIO(s))
    elif isinstance(s, str):
        return load(io.StringIO(s))
    raise ValueError("loads should be given a string or bytes object.")
