from . import abstractEffect
from griffin.math import hsvColor
import time


## Fade effect
#  This effects fades from the current color to the new color in a set amount of time.
#  It can be aborted at any time, and will stop as quick as possible.
#  This effect uses the HsvColor.interpolate function to calculate the in-between colors.
class FadeEffect(abstractEffect.AbstractEffect):
    # @param end_color: HsvColor object, New color that we want to reach.
    # @param fade_time: float, time between the start of this effect and reaching the final color, in seconds.
    def __init__(self, end_color, fade_time):
        super().__init__()
        assert isinstance(end_color, hsvColor.HsvColor)

        self.__end_color = end_color
        self.__fade_time = fade_time

    ## Run the fade effect
    def _run(self):
        start_color = self._getOutput()
        # If the end color equals the start color, we have no work to do.
        if start_color == self.__end_color:
            return

        start_time = time.monotonic()
        while time.monotonic() - start_time < self.__fade_time:
            if self._abort_event.is_set():
                return
            self._setOutput(start_color.interpolate(self.__end_color, (time.monotonic() - start_time) / self.__fade_time))
        self._setOutput(self.__end_color)
