import os
import logging
log = logging.getLogger(__name__.split(".")[-1])


## This class controls the 3 leds on the Atmel board
class DebugLedsHardware():
    __BRIGHTNESS_ON = 255
    __BRIGHTNESS_OFF = 0
    __LEDS = ["ol1", "ol2", "ol3"]

    def __init__(self):
        super().__init__()
        self.__base_path = "/sys/class/leds/opinicus:orange:%s/%s"
        self.__available = self.__checkAvailability()
        self.__turnAllOff()

    ## @brief Check for existence of the brightness file, if it's missing, then the leds are missing
    #  @return Returns True if the led(s) are available, False if not
    def __checkAvailability(self):
        for led in self.__LEDS:
            if not os.path.isfile(self.__base_path % (led, "brightness")):
                return False
        return True

    ## @brief Turns all leds off
    def __turnAllOff(self):
        for led in self.__LEDS:
            self.turnOff(led)

    ## @brief Turns the led on
    #  @param led The led indicator
    def turnOn(self, led):
        if self.__available:
            self.__setBrightness(led, self.__BRIGHTNESS_ON)

    ## @brief Turns the led on
    #  @param led The led indicator
    def turnOff(self, led):
        if self.__available:
            self.__setBrightness(led, self.__BRIGHTNESS_OFF)

    def __setBrightness(self, led, brightness):
        assert led in DebugLedsHardware.__LEDS
        with open(self.__base_path % (led, "brightness"), "w") as f:
            f.write(str(brightness))
