import logging
import time

from griffin import dbusif

log = logging.getLogger(__name__.split(".")[-1])


## This class controls the LED strip that is in the case of the printer
class MarlinLEDDebug:
    # TODO EM-1800 get the maximum brightness from the printer
    __MAX_BRIGHTNESS = {
        "red": 255,
        "green": 255,
        "blue": 255,
        "white": 255,
    }

    def __init__(self):
        self.__printer = dbusif.RemoteObject("printer")
        self.__last_color_cmd = None

    ## Set the brightness of the RGBW leds using the debug M142 Gcode
    #  @param red: float - Red output in range 0.0 to 1.0
    #  @param green: float - Green output in range 0.0 to 1.0
    #  @param blue: float - Blue output in range 0.0 to 1.0
    #  @param white: float - Blue output in range 0.0 to 1.0
    def setBrightness(self, red=None, green=None, blue=None, white=None):
        color_cmd = "M142"
        if red is not None:
            color_cmd += " r%d" % int(red * self.__MAX_BRIGHTNESS["red"])

        if green is not None:
            color_cmd += " g%d" % int(green * self.__MAX_BRIGHTNESS["green"])

        if blue is not None:
            color_cmd += " b%d" % int(blue * self.__MAX_BRIGHTNESS["blue"])

        if white is not None:
            color_cmd += " w%d" % int(white * self.__MAX_BRIGHTNESS["white"])

        # TODO EM-1801 do not use debug calls as an API call
        if self.__last_color_cmd != color_cmd:
            self.__last_color_cmd = color_cmd
            while not self.__printer.debugSendGCodeNoWait(color_cmd):
                time.sleep(0.1)
