## This module allows you to map certain values to other values in different ways.

def __constrained(f, start, end):
    return start + (end - start) * f

## This maps a value to a different range of values on a linear scale.
# @param current_value the current value can be a temperature or time for instance.
# @param start_value the time or temperature when we started for instance
# @param end_value the time or temperature when we expect to be done or will stop for instance
# @param start_of_mapping the first value of the new mapping for instance 0
# @param end_of_mapping the last value to map to like 100 if you map to percentages, etc.
#
# For example:
#
# start_value                  end_value
#    +--------A-------------------+
#             current_value
#
#     start_of_mapping              end_of_mapping
#               +-----B-------------+
# In the drawing above, A is the current value on a linear interval, which is mapped to B on a different linear interval.
def linearMap(current_value, start_value, end_value, start_of_mapping, end_of_mapping):
    delta = end_value - start_value
    if delta == 0:
        return start_of_mapping
    t = (current_value - start_value) / delta
    return __constrained(t, start_of_mapping, end_of_mapping)
