#! /usr/bin/env python3
from collections import deque

## Keep tracks of statistics over a fixed window of a signals that comes in sample by sample
# @param length the sample size of the window to calculate stats over
class MovingWindowStatistics:

    def __init__(self, length):
        self._values = deque(maxlen=length)

    ## @brief process a sample value to be incorporated into the window
    # @returns average, min, max over the window including the currently sent value
    def process(self, value):
        """
        >>> ms = MovingWindowStatistics(10)
        >>> for v in range(10): print(ms.process(v))
        (0.0, 0, 0)
        (0.5, 0, 1)
        (1.0, 0, 2)
        (1.5, 0, 3)
        (2.0, 0, 4)
        (2.5, 0, 5)
        (3.0, 0, 6)
        (3.5, 0, 7)
        (4.0, 0, 8)
        (4.5, 0, 9)
        """
        self._values.append(value)
        stats = sum(self._values)/len(self._values) if self._values else None, \
                min(self._values) if self._values else None, \
                max(self._values) if self._values else None
        return stats

if __name__ == "__main__":
    import doctest
    doctest.testmod()