from griffin import signal
from griffin import thread

from griffin.printer.controller import Controller

import logging

log = logging.getLogger(__name__.split(".")[-1])


class FileHandler:
    def __init__(self):
        self.onFinished = signal.Signal()
        self._controller = Controller.getInstance()

        super().__init__()

    def start(self):
        thread.Thread("FileHandler", self._run).start()

    def _run(self):
        try:
            self.run()
        except:
            log.exception("Exception in file handling")
            # TODO: Proper error handling
            self.onFinished.emit()

    def run(self):
        raise NotImplementedError()

    def cleanup(self):
        pass    # Override in subclass if required.

    def abort(self):
        pass    # Override in subclass, called when the user wants to abort the print

    # @brief Extracts the filename from the full qualified path
    # @return Returns the filename
    def getBasename(self):
        raise NotImplementedError("not implemented in base class")

    ## @return the total filesize in bytes of the (uncompressed) file.
    def getFileSize(self):
        raise NotImplementedError("not implemented in base class")

    def getMetaData(self):
        return {
            "progress": float(-1.0),
            "time_elapsed": int(-1),
            "time_total": int(-1),
        }

    @classmethod
    def getSupportedFileExtensions(cls):
        raise NotImplementedError

    ## @brief Verify if filename is supported by this factory by comparing the filetype
    #  @param filename The filename to check
    #  @return Returns True if the factory supports the handling of the file
    @classmethod
    def isSupportedFile(cls, filename):
        filename = filename.lower()
        for supported_file_type in cls.getSupportedFileExtensions():
            if filename.endswith(supported_file_type.lower()):
                return True
        return False
