from .hotend import Hotend
from ._StringItem import StringItem
from ._NumericItem import NumericItem

class DummyHotend(Hotend):
    _layout = { "static": {
        # Make sure the dummy hotend data has a hotend_cartridge_id, else the rest of the system falls apart.
        "hotend_cartridge_id":  StringItem(0x0e, 10, "hotend cartridge identification string assigned by manufacturer"),

        # Make sure the dummy hotend has a nozzle size, this is needed to start a print.
        "nozzle_size":          NumericItem(0x20, "ff000000", "size of the 'opening' of the nozzle", (lambda x: x / 40.0), (lambda x: x * 40.0), "mm"),

        # Supply a nominal resistance, else we can never heatup this cartridge.
        "nominal_resistance":   NumericItem(0x26, "00fff800", "this cartridges's heater resistance", *Hotend._FLOATING_POINT_PRECISION_NR_OF_DIGITS[2], unit="Ohm"),
    }, "reserved": {} }

    def __init__(self, hotend_index):
        super().__init__(hotend_index, b"DUMMY" + hotend_index.to_bytes(1, "big"), bytes(128))

        self.setValue("hotend_cartridge_id", "?? 0.4")
        self.setValue("nominal_resistance", 23)
        self.setValue("nozzle_size", 0.4)   # Assume a 0.4 nozzle
