from griffin.printer.procedures.procedure import Procedure
from griffin.printer.procedures.diagnostics.determineEndstopStatusStep import DetermineEndstopStatusStep
from griffin.printer.procedures.steps.powerDownSteppersStep import PowerDownSteppersStep

import logging

log = logging.getLogger(__name__.split(".")[-1])


## @brief This procedure retrieves the state of the endstops / limit switches, by sending the M119 command to marlin and
# processing the result
class DetermineEndstopStatusProcedure(Procedure):

    ## @brief Initializes the procedure
    #  @param key The procedure key
    def __init__(self, key):
        super().__init__(key, Procedure.TYPE_MAINTENANCE)
        self.__meta_data = {}

    # Setup the steps for the procedure.
    def setup(self, args):
        # This step will be looping continuously, until ABORT message is received
        m119_step = DetermineEndstopStatusStep("DetermineEndstopStatusStep")
        m119_step.stepDataReady.connect(self.__onNewLimitSwitchData)

        steps = [
            PowerDownSteppersStep("POWER_DOWN_STEPPERS"),
            m119_step
        ]

        self._addStepSequence(steps)
        return True

    ## @brief This function will be called when the step has data available
    def __onNewLimitSwitchData(self, step_data):
        self.__meta_data = step_data

    def getMetaData(self):
        return self.__meta_data
