from griffin.printer.procedures.procedureStep import ProcedureStep
from griffin.printer.controller import Controller
from griffin import signal

import re
import logging

log = logging.getLogger(__name__.split(".")[-1])


## @brief Endless loop of requesting M119 from Marlin for the endstop states
class DetermineEndstopStatusStep(ProcedureStep):

    def __init__(self, key):
        super().__init__(key)
        # M119 returns a string like: 'x_min: OPEN y_max: OPEN z_max: TRIGGERED\nok N528 P15'
        self.__switch_states_re = re.compile(r"^x_min:\s*(?P<x>(\w+))\s*y_max:\s*(?P<y>(\w+))\s*z_max:\s*(?P<z>(\w+))\s*")
        # When M119 response has been matched, the resulting data is send using this signal
        self.stepDataReady = signal.Signal()

    def run(self):
        c = Controller.getInstance()

        c.waitForQueueToBeEmpty(wait_for_motion_controller=True)
        while not self._abort:
            result = c.sendRawGCode("M119", wait=True)
            log.debug("M119 result = %r", result)

            matches = self.__switch_states_re.match(result)

            if matches:
                step_data = {"X": False, "Y": False, "Z": False}
                for endstop, state in matches.groupdict().items():
                    if state != "OPEN":
                        step_data[endstop.upper()] = True
                self.stepDataReady.emit(step_data)
            else:
                log.debug("Unable to match raw data from M119 (%s)", result)

