from griffin.printer.procedures.procedure import Procedure
from griffin.printer.procedures.steps.gotoPositionStep import GotoPositionStep
from griffin.printer.procedures.pre_and_post_print.auto_bed_level_adjust.probeZStep import ProbeZStep
from griffin.printer.procedures.steps.sleepStep import SleepStep
from griffin.printer.buildPlateLocations import BuildPlateLocations
import time
import logging

log = logging.getLogger(__name__.split(".")[-1])

class ProbeProcedure(Procedure):
    def __init__(self, key):
        super().__init__(key, Procedure.TYPE_MAINTENANCE)
        self.__probe_result_storage = []
        self.__last_probe_height = None
        self.__last_probe_timestamp = None
        self.__probe_step = None

    def setup(self, args):
        self.__probe_step = ProbeZStep("PROBING", self.__probe_result_storage)
        self.__probe_step.stepFinished.connect(self.__onProbeFinished)

        steps = [
            GotoPositionStep("GOTO_POS", **args),
            SleepStep("SLEEP", 0.01),
            self.__probe_step,
            GotoPositionStep("MOVE_BED_FOR_SAFE_TRAVEL", z=BuildPlateLocations.getSafeTravelHeight()),
        ]

        self._addStepSequence(steps)
        return True

    ## Callback from the probe step when the probe is finished.
    #  Used to store the result from the probe in the metadata.
    def __onProbeFinished(self):
        if len(self.__probe_result_storage) > 0:
            self.__last_probe_height = self.__probe_result_storage[0].z
        else:
            self.__last_probe_height = 150.0
        self.__probe_result_storage = []
        self.__probe_retry_counter = self.__probe_step.getRetryCount()
        self.__last_probe_timestamp = int(time.monotonic())

    def getMetaData(self):
        if self.__last_probe_timestamp is None:
            return {}
        return {
            "last_probe_height": self.__last_probe_height,
            "loopcount_of_probe": self.__probe_retry_counter,
            "last_probe_timestamp": self.__last_probe_timestamp
        }
