import logging

from griffin.printer.controller import Controller
from griffin.printer.procedures.otherProcedureStep import OtherProcedureStep
from griffin.printer.procedures.pre_and_post_print.applyXyAlignmentStep import ApplyXyAlignmentStep
from griffin.printer.procedures.pre_and_post_print.runChecksBeforePrint import RunChecksBeforePrint
from griffin.printer.procedures.print.switchActiveHotendStep import SwitchActiveHotendStep
from griffin.printer.procedures.procedure import Procedure
from griffin.printer.procedures.queueRawGCodeWaitStep import QueueRawGCodeWaitStep
from griffin.printer.procedures.steps.homeBedStep import HomeBedStep
from griffin.printer.procedures.steps.homeHeadStep import HomeHeadStep
from griffin.printer.procedures.steps.resetPropertyToDefaultStep import ResetPropertyToDefaultStep
from griffin.printer.procedures.waitForQueueEmptyStep import WaitForQueueToBeEmptyStep


log = logging.getLogger(__name__.split(".")[-1])


# This procedure does all the required steps to setup a printer ready for printing.
# This is called from the PRINT procedure.
class PrePrintProcedure(Procedure):
    def __init__(self, key):
        super().__init__(key, Procedure.TYPE_UNKNOWN)

    ## Setup the pre printing procedure.
    #  This creates all the steps needed for the pre printing process.
    #  @param args: dictionary, ignored by this procedure.
    #  @return bool: true on success.
    def setup(self, args):
        c = Controller.getInstance()
        hotend_count = c.getPropertyValue("hotend_count")
        steps = [
            RunChecksBeforePrint("CHECK")
        ]
        for hotend_index in range(hotend_count):
            steps.append(QueueRawGCodeWaitStep("RESET_OFFSET_%d" % (hotend_index), "M218 T%d X0 Y0 Z0" % (hotend_index)))
        steps.append(ApplyXyAlignmentStep("APPLY_XY_CALIBRATION"))
        for hotend_index in range(hotend_count):
            steps.append(OtherProcedureStep("SET_HOTEND_TEMPERATURE_%d" % (hotend_index), "SET_HOTEND_TEMPERATURE", {"hotend_nr": hotend_index, "target": "standby_temperature"}))
        steps += [
            ResetPropertyToDefaultStep("RESET_PRINT_SPEED", "printer", "movement_speed_modifier"),
            HomeBedStep("HOME_BED"),
            HomeHeadStep("HOME_HEAD"),
            SwitchActiveHotendStep("RESET_ACTIVE_HOTEND", "reset"),
            # Note that a lot of pre-printing setup is handled inside the ALIGN_Z_AXIS procedure.
            # This to streamline the actions.
            OtherProcedureStep("BED_LEVEL_DETECTION", "ALIGN_Z_AXIS", duration_estimate_sec=160),
            OtherProcedureStep("PRIME_ALL_NOZZLES", "PRIME_NOZZLE"),
            WaitForQueueToBeEmptyStep("WAITING")
        ]

        self._addStepSequence(steps)
        return True
