import logging

from griffin.printer.procedures.procedure import Procedure
from griffin.printer.procedures.startProcedureStep import StartProcedureStep
from griffin.printer.procedures.wizard.waitMessageStep import WaitMessageStep

log = logging.getLogger(__name__.split(".")[-1])


# @brief Implements the reprint after abort of a print due to failing or manual abort
# If the print procedure is aborted during (pre)print this can be used as a
# finalization step for a print procedure to trigger the possibility of restarting
# the same print again.
# See the procedure class for how finalization is handled and how the flow
# interacts with the original step sequence.
class ReprintAfterAbortProcedure(Procedure):
    def __init__(self, key):
        super().__init__(key, Procedure.TYPE_PRINT)

    def setup(self, args):
        if args is None:
            args = {}

        # The 'reprint' setting enables the 'print' procedure to skip some
        # printing steps.
        args["reprint"] = True

        steps = [
            WaitMessageStep("WAIT_FOR_USER_TO_START_REPRINT", "REPRINT"),
            StartProcedureStep("START_REPRINT", "PRINT", args)
        ]

        self._addStepSequence(steps)
        return True
