import logging

from griffin.datatypes import eventType
from griffin.printer.controller import Controller
from griffin.printer.procedures.print.pauseStep import PauseStep
from griffin.printer.procedures.procedureStep import ProcedureStep


log = logging.getLogger(__name__.split(".")[-1])


## This step does the final resume action of setting the speed and un-pauses the controller.
class ResumeStep(ProcedureStep):
    def __init__(self, key, resume):
        super().__init__(key)
        self.__resume_procedure = resume

    def run(self):
        c = Controller.getInstance()
        eventType.postEvent(eventType.PRINT_RESUMED, [c.getProcedure("PRINT").getMetaData().get("uuid", "")])
        deprime_speed = c.getPropertyValue("deprime_retraction_speed") * 60
        move_speed = c.getPropertyValue("default_travel_speed") * (1 / (c.getPropertyValue("movement_speed_modifier") / 100)) * 60

        saved_state = self.__resume_procedure.getSavedState()

        # reset the position of the filament because it could have been reset by the change material procedure while paused.
        e_pos = saved_state.get("E" + str(saved_state["active_hotend"]), 0) - PauseStep.RETRACT_LENGTH_ON_PAUSE
        c.sendRawGCode("G92 E{0}".format(e_pos))
        # unretract the big part
        c.sendRawGCode("G1 E{E:f} F{F:f}".format(E=e_pos + PauseStep.RETRACT_LENGTH_ON_PAUSE - PauseStep.QUICK_RETRACT_LENGTH_ON_PAUSE, F=deprime_speed))
        # return Z (XY already moved to the right location in resumePrintProcedure).
        c.sendRawGCode("G0 X{X:f} Y{Y:f} Z{Z:f} ".format(**saved_state) + "F{F:f}".format(F=move_speed))
        # Finally restore the speed to what it was before.
        c.sendRawGCode("G0 F{F:f}".format(**saved_state))

        c.getPrintHeadController().getPropertyContainer().get("cooling_fan_speed").set(saved_state.get("cooling_fan_speed", 0))
        c.setPaused(False)

        # reset the saved state
        self.__resume_procedure.setSavedState({})
