import logging

from griffin.printer.controller import Controller
from griffin.printer.procedures.procedureStep import ProcedureStep
from .procedureStep import OutcomeBase

log = logging.getLogger(__name__.split(".")[-1])


# This class implements a step that starts another procedure, without waiting for it to finish!
class StartProcedureStep(ProcedureStep):
    def __init__(self, key, procedure_key, args={}):
        super().__init__(key)

        self.__procedure = None
        self.__procedure_key = procedure_key
        self.__procedure_setup_args = args

    # This function will just start the procedure, if the setup is correctly handled
    def run(self):
        self.__procedure = Controller.getInstance().getProcedure(self.__procedure_key)

        if self.__procedure is None:
            log.error("Other procedure not found: %s", self.__procedure_key)
            return OutcomeBase.Error

        if not self.__procedure.setup(self.__procedure_setup_args):
            log.warning("Failed to setup procedure: %s", self.__procedure_key)
            log.warning("With parameters: %s", self.__procedure_setup_args)
            return OutcomeBase.Error

        self.__procedure.start()

