
from .gotoPositionStep import GotoPositionStep
from griffin.printer.buildPlateLocations import BuildPlateLocations
import logging

log = logging.getLogger(__name__.split(".")[-1])


# Procedure step to move the printer head to a location specified by name.
class GotoLocationStep(GotoPositionStep):

    # @brief When this step is executed the printer head will move to the specified location in the printer.
    # @param location can be "LEFT_FRONT", "RIGHT_FRONT", "CENTER_REAR", etc.
    # @param hotend_nr Indicating the nozzle to use (with relative offsets)
    def __init__(self, key, location, hotend_nr=0):
        # Default init for the step
        super().__init__(key)
        self._hotend_nr = hotend_nr
        self._location = location

    def run(self):
        position = BuildPlateLocations.getInstance().getPosition(self._location, self._hotend_nr)
        assert position is not None, "Unknown location specified: %s" % self._location

        self._x = position.x
        self._y = position.y
        self._z = position.z

        super().run()
