import logging

from griffin.printer.properties.propertyContainer import PropertyContainer
from griffin.printer.procedures.procedureStep import ProcedureStep
from griffin.printer.controller import Controller

log = logging.getLogger(__name__.split(".")[-1])


## A step to set a property to a value.
#  The constructor accepts both a property container as well as a string to indicate the property container. This is to make it work with the machine json file.
class SetPropertyStep(ProcedureStep):
    # @param step_key: The name of this step.
    # @param property_owner: The propertyContainer object, or a string to indicate where to get the propertyContainer from.
    # @param property_key: The name of the property to set.
    # @param property_value: The value to set the property to.
    def __init__(self, step_key, property_owners, property_key, property_value):
        super().__init__(step_key)

        self.__property_owners = property_owners
        self.__property_key = property_key
        self.__property_value = property_value

    # This function is allowed to block as it is called from a separate thread and will
    # not block the rest of the service.
    def run(self):
        for property in self._getPropertiesForThisStep():
            if not property.set(self.__property_value):
                log.warning("Failed to set property %s to value %s", property.getKey(), self.__property_value)

    def _getPropertiesForThisStep(self):
        c = Controller.getInstance()
        property_owner_map = {
            "printer": c.getPropertyContainer(),
            "printer.bed": c.getBed().getPropertyContainer(),
            "printer.head.0": c.getPrintHeadController().getPropertyContainer(),
        }
        for index in range(c.getPropertyValue("hotend_count")):
            property_owner_map["printer.head.0.hotend.%d" % index] = c.getPrintHeadController().getHotendSlot(index).getPropertyContainer()

        property_owners = self.__property_owners
        if callable(property_owners):
            property_owners = property_owners()

        if not isinstance(property_owners, list):
            property_owners = [property_owners]
        result = []
        for property_owner in property_owners:
            if isinstance(property_owner, PropertyContainer):
                property_container = property_owner
            elif property_owner in property_owner_map:
                property_container = property_owner_map[property_owner]
            else:
                raise RuntimeError("Unknown property owner: %s in SetProcedureStep" % (property_owner))
            result.append(property_container.get(self.__property_key))
        return result
