from griffin.printer.procedures.procedureStep import ProcedureStep
from griffin.math import mapping
import time
import logging

log = logging.getLogger(__name__.split(".")[-1])

## @brief Allows to sleep for a certain amount of time, which can be aborted
class SleepStep(ProcedureStep):

    ## @brief Setup the sleep period
    #  @param delay The sleep time in second
    def __init__(self, key, delay=0):
        super().__init__(key)
        self.__delay = delay
        self.__end_time = time.monotonic()

    ## @brief Set the delay time
    #  @param delay The sleep time in second
    def setTime(self, delay):
        self.__delay = delay

    def run(self):
        self.__end_time = time.monotonic() + self.__delay
        while  not self.isFinished() and not self._abort:
            # Using this small sleep allows for delay of 0.1s
            time.sleep(0.05)

    ## @brief Verifies if the time has run passed the end time (base on monotonic time)
    #  @return Returns True if the current monotonic time has surpassed the end time
    def isFinished(self):
        return time.monotonic() >= self.__end_time

    ## Return the metadata for this step.
    #  For the sleep step, this means the total timeout, and the time remaining
    #  @return a dictionary containing the "TOTAL" and "REMAINING" keys, both containing a float in seconds.
    def getMetaData(self):
        progress = mapping.linearMap(time.monotonic(), self.__end_time - self.__delay, self.__end_time, 0, 1)
        return {
            "TOTAL": self.__delay,
            "REMAINING": max(0, self.__end_time - time.monotonic()),
            "progress": progress
        }
