import logging
from griffin.printer.procedures.procedureStep import ProcedureStep

log = logging.getLogger(__name__.split(".")[-1])

## @brief Store a value at some storage_key in a dictionary
# You want to use this step to evaluate and calculate a value at runtime, when the Procedure is being executed and stepping through its steps.
#
# @param key str the key (i.e. name) of the step
# @param storage dict in which the data is to be stored
# @param storage_key str the key in the storage-dict in which the value is to be stored. Code does storage[storage_key] = ...
# @param evaluator a callable that determines the value that must be stored.
class StoreValueStep(ProcedureStep):
    def __init__(self, key, storage, storage_key, evaluator):
        super().__init__(key)

        self.storage = storage
        self.storage_key = storage_key

        assert callable(evaluator)
        self.evaluator = evaluator

    def run(self):
        value = self.evaluator()
        log.info("Evaluator of '{k}' storing value {val} at key '{sk}'".format(k=self.getKey(), val=value, sk=self.storage_key))
        self.storage[self.storage_key] = value
