import logging

from griffin.printer.controller import Controller
from griffin.printer.heatableObject.heatableHotend import HeatableHotendSlot
from griffin.printer.procedures.temperature.setHeatableObjectTemperatureStep import SetHeatableObjectTemperatureStep

log = logging.getLogger(__name__.split(".")[-1])


class SetHotendTemperatureStep(SetHeatableObjectTemperatureStep):
    def __init__(self, key, target_configuration=None):
        self.__hotend_index = 0
        if target_configuration is not None:
            self.__hotend_index = HeatableHotendSlot.deriveHotendNr(**target_configuration)

        super().__init__(key, target_configuration, Controller.getInstance().getPrintHeadController().getHotendSlot(self.__hotend_index))

    ## Function to update the selected hotend from a set of parameters.
    #  Called from the waitForHotendTemperatureProcedure to configure the proper hotend
    #  @param **args: keyword arguments that are passed to the deriveHotendNr function.
    def updateTargetHotend(self, **args):
        self.__hotend_index = HeatableHotendSlot.deriveHotendNr(**args)
        self._updateHeatableObject(Controller.getInstance().getPrintHeadController().getHotendSlot(self.__hotend_index))
        return True

    ## Get the hotend index for this SetHotendTemperatureStep
    #  @result integer containing the hotend index
    def getHotendIndex(self):
        return self.__hotend_index
