import logging

from griffin.printer.controller import Controller
from griffin.printer.faultHandler import FaultHandler, FaultCode

from .waitForHeatableObjectsStep import WaitForHeatableObjectsStep


log = logging.getLogger(__name__.split(".")[-1])


## Class to set a bed temperature and wait for it to be reached.
class WaitForBedTemperatureStep(WaitForHeatableObjectsStep):
    ## Seconds to wait for bed to reach temperature setpoint
    __DEFAULT_BED_TEMP_TIMEOUT = 30 * 60

    ## Create a new WaitForBedTemperatureStep.
    #  @param key: string, name of this step.
    #  @param target_temperature_configuration: optional, dictionary, containing the key/values that are used to configure this step.
    #                               see HeatablePrintBed.deriveTemperature functions for details.
    def __init__(self, key, target_temperature_configuration=None):
        super().__init__(key, target_temperature_configuration, step_timeout=self.__DEFAULT_BED_TEMP_TIMEOUT)

        self._setHeatableObjects([Controller.getInstance().getBed()])

    def _onStepTimeout(self):
        FaultHandler.critical("Waiting for bed temperature is taking too long", FaultCode.BED_TEMPERATURE_TIMEOUT)  # For displaying
