from griffin.printer.procedures.procedureStep import ProcedureStep
from griffin.printer.controller import Controller
from griffin.thread import Event
import logging

log = logging.getLogger(__name__.split(".")[-1])

## This class receives and stores the nr of print cores selected in the value_store
class SelectPrintCoresToCleanStep(ProcedureStep):
    def __init__(self, key, value_store):
        super().__init__(key)
        self.__value_store = value_store
        self.__message_received_event = Event()

    def run(self):
        from .nozzleCleaning import NozzleCleaning
        self.__message_received_event.clear()

        ## empty out the list for repeated iterations trough the wizard.
        while self.__value_store:
            self.__value_store.pop()

        # skip this step for a single printcore printer.
        if Controller.getInstance().getPropertyValue("hotend_count") < 2:
            self.__value_store.append(NozzleCleaning.DATA_STORAGE_PRINTCORE_PREFIX + "0")
            return

        Controller.getInstance().setPropertyValue("interaction_required", True)

        while not self.__message_received_event.is_set() and not self._abort:
            self.__message_received_event.wait(self.TIME_OUT_WAIT_FINISHED_EVENT)

        Controller.getInstance().setPropertyValue("interaction_required", False)

    def receiveMessage(self, message):
        log.info("Message: %s", message)
        from .nozzleCleaning import NozzleCleaning

        if str(message).startswith(NozzleCleaning.DATA_STORAGE_PRINTCORE_PREFIX):
            self.__value_store.append(str(message))
            self.__message_received_event.set()
        elif message == "ALL":
            self.__value_store += [NozzleCleaning.DATA_STORAGE_PRINTCORE_PREFIX + "%d" % n for n in range(Controller.getInstance().getPropertyValue("hotend_count"))]
            self.__message_received_event.set()
