(function() {
  var api_base_path = '/api/v1';

  var detectWifiSetup = function() {
    var url = api_base_path + '/printer/network/wifi';
    $.ajax({
      url: url,
      type: 'GET',
      contentType: 'application/json',
      dataType: 'json',
      complete: function(data) {
        var mode= data.responseJSON["mode"];
        if(mode == "WIFI SETUP") {
          $('#wifi-setup-div').removeClass("hidden-wifi-setup")
        }
        return;
      }
    });
  };

  var detectAndRedirectLanguage = function() {
    var url = api_base_path + '/system';

    $.ajax({
      url: url,
      type: 'GET',
      contentType: 'application/json',
      dataType: 'json',
      complete: function(data) {
        if(data.responseJSON == undefined) {
          window.location = "default.html";
        }

        var language = data.responseJSON["language"];
        var path = language;

        var available_languages = ["en"];

        language = _.find(available_languages, function(current_language) { return current_language == language; });

        if(language == "en" || language == undefined) {
          path = "";
        }

        window.location = path + "/default.html";
      }
    });
  };

  var connect = function(ssid, password, failed, connected) {
    var times_tried = 0;
    var retry_duration = 3000;
    var url = api_base_path + '/printer/network/wifi_networks/' + encodeURIComponent(ssid);

    $.ajax({
      url: url,
      type: 'PUT',
      contentType: 'application/json',
      dataType: 'json',
      data: JSON.stringify({'passphrase': password}),
      complete: function(data) {
        if(data.responseText.indexOf('No network found') > -1) {
          failed();
        }
      }
    });

    var connectionFailed = function() {
      if (times_tried > 5) {
        connected();
        return;
      }

      times_tried++;
    };

    setInterval(function() {
      $.ajax({
        url: api_base_path + "/printer/network/wifi",
        dataType: 'json',
        success: function(data) {
          if (!data.connected || data.ssid != ssid) {
            connectionFailed();
            return;
          }

          connected();
        },
        error: function(xhr, txt, err) {
          // The printer is "probably connected" when the hotspot is no longer available
          // which means that generic HTTP errors are *not* valid for inferring that the connection
          // succeeded. Therefore, checking the XHR ready state is probably the best
          // option.
          if (xhr.readyState == 0) { 
            connected();
            return;
          }

          connectionFailed();
        }
      });
    }, retry_duration);
  };

  $('.js-show-password, .js-password').on('click', function(e) {
    if($(e.currentTarget).is(':checked')) {
      var $inputs = $('input[type=password], .js-password');
      $inputs.addClass("js-password");
      $inputs.attr('type', 'text');
    } else {
      $('input[type=password], .js-password').attr('type', 'password');
    }
  });

  if(window.location.search.replace("?", "") == "manual=1") {
    $("a.js-retype-password").attr("href", "manual.html")
  }

  $('#js-printer-name-form').on('submit', function(e) {
    e.preventDefault();
    var $printer_name = $('#js-printer-name');
    var name = $printer_name.val();
    if(name == "") {
      $('#js-printer-name-form').addClass("error-state")
    } else {
      $.ajax({
        url: api_base_path + '/system/name',
        type: 'PUT',
        contentType: 'application/json',
        dataType: 'json',
        data: JSON.stringify(name)
      }).complete(function(data) {
        var result = JSON.parse(data.responseText).return_value;
        if(result == 1) {
          window.location.href = '../wifi/select-network.html';
        } else {
          $('#js-printer-name-form').addClass("error-state")
        }
      });
    }
  });

  $('.js-select-ssid').on('.js-network click', function(e) {
    var $current = $(e.target);
    var ssid = $current.data('ssid');
    var secure = ($current.data('needs-password') == true);
    setCookie("ssid", ssid);

    if(secure) {
      window.location.href = '../wifi/enter-password.html';
    } else {
      $(".js-select-ssid").addClass("loading");

      connect(ssid, "", function() {
        window.location.href = '../wifi/failed.html';
      }, function() {
        $('.mod-wifi-setup').addClass('probably-connected');
      })
    }
  });

  $('#js-select-country .js-submit-country').on('click', function(e) {
    $('#js-select-country').addClass('loading');

    var country_code = $('#js-select-country option:selected').data("code");
    $.ajax({
      url: api_base_path + '/system/country',
      type: 'PUT',
      contentType: 'application/json',
      dataType: 'json',
      data: JSON.stringify(country_code)
    }).always(function() {
      window.location.href = '../wifi/name.html';
    });
  });

  requestNetworks = function() {
    $('.js-select-ssid').addClass('loading');

    setTimeout(function() {
      var ssid_list = $('.js-select-ssid ul.ssids').empty();

      $.getJSON(api_base_path + '/printer/network/wifi_networks', function(wifi_networks) {
        if(wifi_networks.length > 0) {
          for(var i in wifi_networks) {
            var network = wifi_networks[i];

            var name = network.SSID;
            var strength_percentage = network.strength;
            var needs_password = network.security_required;

            ssid_list.append('<li class="js-network strength-' + strength_percentage + '" data-ssid="' + name + '" data-needs-password="' + needs_password + '">' + name + '</li>');
          }
        } else {
          $('.found-networks').hide();
          $('.found-no-networks').removeClass('hidden');
        }

        $('.js-select-ssid').removeClass('loading');
      });
    }, 2000);
  };

  var submitPassword = function() {
    $('.js-wifi-password').addClass('loading');
    connect(getCookie("ssid"), $('#js-password').val(), function() {
      window.location.href = '../wifi/failed.html';
    }, function() {
      $('.mod-wifi-setup').addClass('probably-connected');
    })
  }

  $('.js-submit-password').click(function() {
      submitPassword();
    }
  );

  $('.js-submit-password-manual').click(function() {
      var ssid = $('#js-ssid').val();
      setCookie("ssid", ssid);
      submitPassword();
    }
  );


  if($('.js-select-ssid').length > 0) {
    requestNetworks();
  }

  if($('.js-detect-and-redirect-to-language').length > 0) {
    detectAndRedirectLanguage();
  }

  if($('.js-detect-wifi-setup').length > 0) {
    detectWifiSetup();
  }
})();
